/******************************************************************************
 * boot_logo.c |
 *------------
 *
 * A quick program to read in an image and create the corresponding
 * linux_logo.h file for use as the framebuffer console boot logo.
 *
 * Written by Kevin R. Sandy <kevin@z-sol.com>, tty666  tty666@zophia.com.br
 *
 * low_color.h taken from fblogo-0.1.
 *
 * This program is released under the GNU GPL license.
 *
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include "low_color.h"

#define MAX_COLORS 256
#define MAX_IMAGE_SIZE 786432				// Enough to hold a 1024x768 image

int errno;

/*----- Structs to hold image file info ---------*/

typedef struct struct_palette
{
	unsigned char red[256];
	unsigned char green[256];
	unsigned char blue[256];
} struct_palette;

typedef struct struct_picture
{
	struct_palette palette;
	unsigned int height;
	unsigned int width;
	unsigned long int bytes_in_image;		// The number of bytes used to store the image
	unsigned long int pixels_in_image;		// The number of pixels in the image
	unsigned char image_data[MAX_IMAGE_SIZE];
} struct_logo;

struct_logo logo;


/*----- Function prototypes -------------------*/

int usage(void);
int load_image_file(const char *);
int write_logo_header(const char *);
int write_image_data(FILE *, unsigned char *);
int write_palette_info(FILE *, char *, unsigned char *);
int read_pcx_file(FILE *);



/*----- main() --------------------------------------------------------------*/

int main(int argc, char **argv)
{
	char *image_file_name;
	char *header_file_name;

	if ( argc != 3 )	// There should be two filenames
	{			// passed. If not, print usage
		usage();	// info and exit.

		return 1;
	}

	image_file_name = argv[1];
	header_file_name = argv[2];

	if ( !(load_image_file(image_file_name)) )
	{
		fprintf(stderr, "Exiting due to errors.\n\n");

		return 1;
	}
	else if ( !(write_logo_header(header_file_name)) )
	{
		fprintf(stderr, "Exiting due to errors.\n\n");

		return 1;
	}
	else
	{
		fprintf(stdout, "\n");
		fprintf(stdout, "Successfully created %s from %s!\n", header_file_name, image_file_name);
		fprintf(stdout, "Be sure to edit /usr/src/linux/drivers/video/fbcon.c and\n");
		fprintf(stdout, "change LOGO_H and LOGO_W to reflect the new height and width.\n");
		fprintf(stdout, "#define LOGO_H %u\n", logo.height);
		fprintf(stdout, "#define LOGO_W %u\n", logo.width);
		fprintf(stdout, "\n");
		fprintf(stdout, "Then, copy %s to /usr/include/linux/linux_logo.h, rebuild\n", header_file_name);
		fprintf(stdout, "your kernel, enjoy!\n");
		fprintf(stdout, "\n");

		return 0;
	}
}



/*----- usage() -------------------------------------------------------------*/

int usage(void)
{
	fprintf(stderr, "\n");
	fprintf(stderr, "Usage: boot_logo <input_image> <output_header>\n");
	fprintf(stderr, "\n");

	return 1;
}



/*----- load_image_file() ---------------------------------------------------*/

int load_image_file(const char *image_file_name)
{
	FILE *image_file;

	image_file = fopen(image_file_name, "rb");

	if ( !image_file )
	{
		fprintf(stderr, "\n");
		fprintf(stderr, "Error opening %s for reading:\n", image_file_name);
		fprintf(stderr, "\t%s\n", strerror(errno));
		fprintf(stderr, "\n");
		return 0;
	}

	read_pcx_file(image_file);

	fclose(image_file);					// Done with the image file, so close it.

	return 1;
}



/*----- write_logo_header() -------------------------------------------------*/

int write_logo_header(const char *header_file_name)
{
	FILE *header_file;

	header_file = fopen(header_file_name, "wb");

	if ( !header_file )
	{
		fprintf(stderr, "\n");
		fprintf(stderr, "Error opening %s for writing:\n", header_file_name);
		fprintf(stderr, "\t%s\n", strerror(errno));
		fprintf(stderr, "\n");
		return 0;
	}

	fprintf(header_file, "#if LINUX_LOGO_COLORS == 214\n\n");

	write_palette_info(header_file, "red", (char *)logo.palette.red);
	write_palette_info(header_file, "green", (char *)logo.palette.green);
	write_palette_info(header_file, "blue", (char *)logo.palette.blue);
	write_image_data(header_file, (char *)logo.image_data);

	fprintf(header_file, "#endif\n\n");
	fprintf(header_file, "%s", low_color_logo);

	fclose(header_file);

	return 1;
}



/*----- write_image_data() --------------------------------------------------*/

int write_image_data(FILE *header_file, unsigned char *image_data)
{
	unsigned int ctr = 0;

	fprintf(header_file, "unsigned char linux_logo[] __initdata = {\n");

	while ( ctr < logo.pixels_in_image )
	{
		fprintf(header_file, "0x%2.2hx", image_data[ctr++] + 0x20);
		if ( ctr == logo.pixels_in_image )
		{
			fprintf(header_file, "\n};\n\n");
		}
		else if ( ctr % 8 == 0 )
		{
			fprintf(header_file, ",\n");
		}
		else
		{
			fprintf(header_file, ", ");
		}
	}

	return 1;
}



/*----- write_palette_info() ------------------------------------------------*/

int write_palette_info(FILE *header_file, char *color, unsigned char *palette)
{
	unsigned int ctr;

	fprintf(header_file, "unsigned char linux_logo_%s[] __initdata = {\n", color);
	ctr = 0;
	while ( ctr < 256 )
	{
		fprintf(header_file, "0x%2.2hx", palette[ctr++]);
		if ( ctr == 256 )
		{
			fprintf(header_file, "\n};\n\n");
		}
		else if ( ctr % 8 == 0 )
		{
			fprintf(header_file, ",\n");
		}
		else
		{
			fprintf(header_file, ", ");
		}
	}

	return 1;
}



/*---------------------------------------------------------------------------*/
/*----- Begin PCX specific functions ----------------------------------------*/
/*---------------------------------------------------------------------------*/



/*----- read_pcx_file() -----------------------------------------------------*/

int read_pcx_file(FILE *pcx_file)
{
	unsigned char temp_buffer;
	unsigned int ctr, image_ctr;
	unsigned int rle_run_length;

	fseek(pcx_file, -768L, SEEK_END);			// The last 768 bytes represent the color
	logo.bytes_in_image = ftell(pcx_file) - 128;		// palettes.  The first 128 are headers.

	fseek(pcx_file, 8L, SEEK_SET);
	fscanf(pcx_file, "%2c", &logo.width);
	logo.width++;
	fscanf(pcx_file, "%2c", &logo.height);
	logo.height++;

	fseek(pcx_file, 128L, SEEK_SET);			// We don't need the headers, so skip them.

	for ( ctr = 0, image_ctr = 0; ctr < logo.bytes_in_image; ctr++)
	{
		temp_buffer = fgetc(pcx_file);

		if ( temp_buffer >= 192 )			// If the byte is >= 192, it represents
		{						// the length of a run, so replicate the
			rle_run_length = temp_buffer - 192;	// next byte x - 192 times.
			temp_buffer = fgetc(pcx_file);
			ctr++;					// We've read from the file, so increment
								// the loop counter.
			while ( rle_run_length-- > 0 )
			{
				logo.image_data[image_ctr++] = temp_buffer;
			}
		}
		else						// Otherwise, it represents the next pixel.
		{
			logo.image_data[image_ctr++] = temp_buffer;
		}
	}

	logo.pixels_in_image = --image_ctr;

	for ( ctr = 0; ctr < 256; ctr++)			// Next, grab the 8bpp color palette.
	{
		logo.palette.red[ctr] = fgetc(pcx_file);
		logo.palette.green[ctr] = fgetc(pcx_file);
		logo.palette.blue[ctr] = fgetc(pcx_file);
	}

	return 1;
}



/*****************************************************************************/
